from typing import Any
import sqlite3
import re
from resql.core.errors import PydbInvalidTableName

regex = re.compile("[A-z0-9]+")

class ReSql:
    default_row:str = "DEFAULT_RESQL_TABLE"

    def __init__(self, path:str=":memory:", props:dict={}):
        self.connection = sqlite3.connect(path)
        self.cursor = self.connection.cursor()
        self.cursor.execute("CREATE TABLE IF NOT EXISTS DEFAULT_RESQL_TABLE (ID TEXT, value TEXT)")

    def commit(self):
        self.connection.commit()
        return 

    def insert(self,key:str,value:Any, table:str=default_row):
        self.cursor
        self.cursor.execute(f"INSERT INTO {table} (ID, value) VALUES (?, ?)", (key, value))
        self.commit()
        return {"status":"success"}

    def find(self,key:str, table:str=default_row):
        self.cursor.execute(f"SELECT * FROM {table} WHERE ID = ?", (key,))
        resp = self.cursor.fetchone()
        self.commit()
        return resp[1] if resp != None else None

    def get(self,key:str, table:str=default_row):
        self.cursor.execute(f"SELECT * FROM {table} WHERE ID = ?", (key,))
        resp = self.cursor.fetchone()
        self.commit()
        return resp[1] if resp != None else None

    def all(self, table:str=default_row):
        self.cursor.execute(f"SELECT * FROM {table}")
        resp = self.cursor.fetchall()
        self.commit()
        return resp if resp != None else None

    def wipe(self, table:str=default_row):
        self.cursor.execute(f"delete from {table}")
        self.commit()
        return {"status":"success"}

    def delete(self,key:str, table:str=default_row):
        self.cursor = self.connection.cursor()
        self.cursor.execute(f"DELETE FROM {table} WHERE ID = ?", (key,))
        self.commit()
        return {"status":"success"}

    def create_table(self,name:str):
        if regex.match(name) != None and regex.match(name)[0] == name:
            self.cursor.execute(f"CREATE TABLE IF NOT EXISTS {name} (ID TEXT, value TEXT)")
        else:
            raise PydbInvalidTableName("Invalid table name")
        self.commit()
        return {"status":"success","table":name}

    def run(self,sql:str):
        self.cursor.execute(sql)
        self.commit()
    
    def db_cursor(self):
        return self.cursor



class PydbError(Exception):
    """
    Base class for all exceptions in pydb.
    """
    def __init__(self, message):
        self.message = message
    
    def __str__(self):
        return "pydb error: " + self.message
    
class PydbInvalidTableName(PydbError):
    """
    Raised when the table name is invalid.
    """
    def __init__(self, message):
        self.message = message
    
    def __str__(self):
        return self.message
    


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['resql', 'resql.core']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'resql',
    'version': '0.1.5',
    'description': 'A beginner friendly database',
    'long_description': '# <p align="center"> resql: Database For Beginners </p>\n\n## Quick Examples\n<br>\n\nimports\n\n```\npip install resql\n```\n\n\n\n```python\nimport resql\n\ndatabase = resql.ReSql("mydb.db")\n# default is in :memory:\n```\n\n<br>\n\n## .create_table(name)\n\ncreates a database table.\n\n```python\ndatabase.create_table("newtable")\n# {\'status\': \'success\', \'table\': \'newtable\'}\n```\n\n<br>\n\n\n## .insert(key, value, [table])\n\ninsert into given table.\n\n```python\ndatabase.insert("key","value")\n# {\'status\': \'success\'}\n```\n\n<br>\n\n\n## .all([table])\n\nfind data from given table.\n\n```py\ndatabase.all()\n# [(\'key\',\'value\')]\n```\n\n<br>\n\n## .find(key, [table]) or .get(key, [table])\n\nfind data from given table.\n\n```py\ndatabase.find("key")\n# value\n```\n\n<br>\n\n## .wipe([table])\n\nwipe complete table.\n\n\n```py\ndatabase.wipe()\n# {\'status\': \'success\'}\n```\n\n<br>\n\n## .run(sql)\n\ndirectly run sql query.\n\n```py\ndatabase.run("SELECT * FROM table")\n# cursor\n```\n\n<br>\n\n## .delete(key, [table])\n\ndelete key from table.\n\n```py\ndatabase.delete("key")\n# value\n```\n\n\nthanks to [avonryle#2022](https://github.com/avonryle) for letting me make the python version of [ByteDatabase](https://github.com/cloudteamdev/ByteDatabase)\n\n\n\n',
    'author': 'ResetXD',
    'author_email': 'resetwastakenwastaken@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ResetXD/resql',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
